@@ -1,41 +1,43 @@
 import pandas as pd
-# from pandas_ta.utils import get_offset, non_zero_range, verify_series
 import numpy as np
 import pandas_ta as ta
 
 
-# def stochastic_oscillator(dataframe, k=14, d=3, smooth_k=3, mamode='sma'):
-#     """ Function to calculate stochastic oscillator """
-#     try:
-#         # Calculate the rolling lowest low and highest high over the K period
-#         k = k if k and k > 0 else 14
-#         d = d if d and d > 0 else 3
-#
-#         smooth_k = smooth_k if smooth_k and smooth_k > 0 else 3
-#         _length = max(k, d, smooth_k)
-#
-#         high = verify_series(dataframe["high"], _length)
-#         low = verify_series(dataframe["low"], _length)
-#         close = verify_series(dataframe["close"], _length)
-#
-#         if high is None or low is None or close is None: return
-#
-#         # Calculate Result
-#         lowest_low = low.rolling(k).min()
-#         highest_high = high.rolling(k).max()
-#
-#         stoch = 100 * (close - lowest_low)
-#         stoch /= non_zero_range(highest_high, lowest_low)
-#         stoch_k = stoch.rolling(smooth_k).mean()
-#         stoch_d = stoch_k.rolling(d).mean()
-#
-#         new_df = pd.DataFrame()
-#         new_df["k"] = stoch_k
-#         new_df["d"] = stoch_d
-#         return new_df
-#     except Exception as e:
-#         print("Exception in calculating stochastic oscillator : {}".format(e))
-#         return None
+def stochastic_oscillator(dataframe, k=14, d=3, smooth_k=3, mamode='sma'):
+    """ Function to calculate stochastic oscillator """
+    try:
+        if not all(col in dataframe.columns for col in ["high", "low", "close"]):
+            return None
+
+        # Calculate the rolling lowest low and highest high over the K period
+        k = k if k and k > 0 else 14
+        d = d if d and d > 0 else 3
+        smooth_k = smooth_k if smooth_k and smooth_k > 0 else 3
+
+        high = dataframe["high"]
+        low = dataframe["low"]
+        close = dataframe["close"]
+
+        # Calculating Lowest Low and Highest High
+        lowest_low = low.rolling(k, min_periods=1).min()
+        highest_high = high.rolling(k, min_periods=1).max()
+
+        _length = max(k, d, smooth_k)
+
+        stoch = 100 * (close - lowest_low) / (highest_high - lowest_low).replace(0, np.nan)
+
+        if mamode.lower() == 'ema':
+            stoch_k = stoch.ewm(span=smooth_k, adjust=False).mean()
+            stoch_d = stoch_k.ewm(span=d, adjust=False).mean()
+        else:  # default to SMA
+            stoch_k = stoch.rolling(window=smooth_k, min_periods=1).mean()
+            stoch_d = stoch_k.rolling(window=d, min_periods=1).mean()
+
+        new_df = pd.DataFrame({"k": stoch_k, "d": stoch_d})
+        return new_df
+    except Exception as e:
+        print(f"Exception in calculating stochastic oscillator : {e}")
+        return None
 
 
 def volume_flow_indicator(dataframe, length, coef, vcoef, signal_length, smooth_vfi=False):
@@ -101,42 +103,42 @@ def williams_fractal(dataframe, fractal_window=2):
         pass
 
 
-# def mcgd(close, length=None, offset=None, c=None, **kwargs):
-#     """ Indicator: McGinley Dynamic Indicator """
-#     # Validate arguments
-#     length = int(length) if length and length > 0 else 10
-#     c = float(c) if c and 0 < c <= 1 else 1
-#     close = verify_series(close, length)
-#     offset = get_offset(offset)
-#
-#     if close is None: return
-#
-#     # Calculate Result
-#     close = close.copy()
-#
-#     def mcg_(series):
-#         denom = (c * length * (series.iloc[1] / series.iloc[0]) ** 4)
-#         series.iloc[1] = (series.iloc[0] + ((series.iloc[1] - series.iloc[0]) / denom))
-#         return series.iloc[1]
-#
-#     mcg_cell = close[0:].rolling(2, min_periods=2).apply(mcg_, raw=False)
-#     mcg_ds = close[:1]._append(mcg_cell[1:])
-#
-#     # Offset
-#     if offset != 0:
-#         mcg_ds = mcg_ds.shift(offset)
-#
-#     # Handle fills
-#     if "fillna" in kwargs:
-#         mcg_ds.fillna(kwargs["fillna"], inplace=True)
-#     if "fill_method" in kwargs:
-#         mcg_ds.fillna(method=kwargs["fill_method"], inplace=True)
-#
-#     # Name & Category
-#     mcg_ds.name = f"MCGD_{length}"
-#     mcg_ds.category = "overlap"
-#
-#     return mcg_ds
+def mcgd(close, length=10, offset=0, c=0.6, **kwargs):
+    """ Indicator: McGinley Dynamic Indicator """
+    try:
+        close = pd.Series(close).astype(float)
+        length = int(length) if length and length > 0 else 10
+        c = float(c) if c and c > 0 else 0.6
+
+        mcg = [close.iloc[0]]  # seed with first value
+
+        for i in range(1, len(close)):
+            prev = mcg[-1]
+            denom = c * length * (close.iloc[i] / prev) ** 4 if prev != 0 else c * length
+            mcg_val = prev + (close.iloc[i] - prev) / denom
+            mcg.append(mcg_val)
+
+        mcg_series = pd.Series(mcg, index=close.index)
+
+        # Apply offset if provided
+        if offset:
+            mcg_series = mcg_series.shift(offset)
+
+        # Handle fills
+        if "fillna" in kwargs:
+            mcg_series.fillna(kwargs["fillna"], inplace=True)
+        if "fill_method" in kwargs:
+            mcg_series.fillna(method=kwargs["fill_method"], inplace=True)
+
+        # Metadata
+        mcg_series.name = f"MCGD_{length}"
+        mcg_series.category = "overlap"
+
+        return mcg_series
+
+    except Exception as e:
+        print(f"Exception in calculating McGinley Dynamic: {e}")
+        return None
 
 
 def squeeze_momentum(dataframe, bb_length=20, bb_mult=2.0, kc_length=20, kc_mult=1.5, use_true_range=True):
