@@ -106,7 +106,7 @@ def save_lt_order(app_db_cursor, redis_cursor, orders_list, tradingsymbol, quant
         app_db_cursor["lt_orders"].insert_one(lt_order_dict)
 
         # Converting Dict to String
-        redis_dict = {k: str(v) if not isinstance(v, (int, float, None)) else v for k, v in order_dict.items()}
+        redis_dict = {k: str(v) if not isinstance(v, (int, float)) else v for k, v in order_dict.items()}
 
         # redis_dict["quantity_left"] = str(quantity_left)
 
