@@ -136,6 +136,19 @@ def convert_to_trades(orders_list, market_type, order_exit_levels, mode, broker)
                     trade_dict["strategy_id"] = ObjectId(order["strategy_id"])
                     trade_dict["request_id"] = ObjectId(order["request_id"])
 
+                # Adding Indicators Values in the Calculated Trade Dict
+                for input_key in order.keys():
+                    if input_key.startswith("underlying_"):
+                        trade_dict[input_key] = order[input_key]
+
+                for input_key in order.keys():
+                    if input_key.startswith("options_"):
+                        trade_dict[input_key] = order[input_key]
+
+                for input_key in order.keys():
+                    if input_key.startswith("futures_"):
+                        trade_dict[input_key] = order[input_key]
+
             else:
                 if market_type == "spot" or market_type == "futures":
                     if order["position_type"] == "SHORT":
