@@ -10,6 +10,20 @@ import pandas_ta as TA
 orders_url = config.get('orders_url')
 
 
+def get_symbol_details(db_cursor, symbol):
+    """ Function to load and get symbols dict """
+    try:
+        symbol_dict = db_cursor['all_symbols'].find_one({"name": symbol})
+        if symbol_dict:
+            return symbol_dict
+        else:
+            print("No Such Symbol Found")
+            return None
+    except Exception as e:
+        print(f"Exception in Getting Symbols Dict : {e}")
+        return None
+
+
 def change_granularity(data_df, granularity):
     """ Function to Change Granularity for Provided Dataframe """
     try:
@@ -375,6 +389,7 @@ def calculate_rolling_swing(df):
 def apply_indicators(dataframe, indicators_dict):
     """ Function to apply indicators on the dataframe provided """
     try:
+
         if "ATR" in indicators_dict:
             atr_values = TA.atr(dataframe['high'], dataframe['low'], dataframe['close'], length=indicators_dict["ATR"]["length"],
                                 mamode=indicators_dict["ATR"]["mamode"])
@@ -520,11 +535,11 @@ def apply_indicators(dataframe, indicators_dict):
             dataframe["ecr"] = ecr_values.tolist()
 
         if "STOCH" in indicators_dict:
-            # stoch_df = stochastic_oscillator(dataframe=dataframe, k=indicators_dict["STOCH"]["k"], d=indicators_dict["STOCH"]["d"],
-            #                                  smooth_k=indicators_dict["STOCH"]["smooth"])
-
-            stoch_df = TA.stoch(high=dataframe["high"], low=dataframe["low"], close=dataframe["close"],
-                                k=indicators_dict["STOCH"]["k"], d=indicators_dict["STOCH"]["d"], smooth_k=indicators_dict["STOCH"]["smooth"], offset=0)
+            stoch_df = stochastic_oscillator(dataframe=dataframe, k=indicators_dict["STOCH"]["k"], d=indicators_dict["STOCH"]["d"],
+                                             smooth_k=indicators_dict["STOCH"]["smooth"])
+            #
+            # stoch_df = TA.stoch(high=dataframe["high"], low=dataframe["low"], close=dataframe["close"],
+            #                     k=indicators_dict["STOCH"]["k"], d=indicators_dict["STOCH"]["d"], smooth_k=indicators_dict["STOCH"]["smooth"], offset=0)
             stoch_k = round(stoch_df.iloc[:, 0], 2).fillna(0).values
             stoch_d = round(stoch_df.iloc[:, 1], 2).fillna(0).values
             dataframe["stoch_k"] = stoch_k
@@ -562,8 +577,6 @@ def apply_indicators(dataframe, indicators_dict):
         if "MACD" in indicators_dict:
             macd_df = TA.macd(close=dataframe[indicators_dict["MACD"]["source"]], fast=indicators_dict["MACD"]["fast_length"],
                               slow=indicators_dict["MACD"]["slow_length"], signal=indicators_dict["MACD"]["signal_smoothing"], talib=True)
-            # macd_df = custom_macd(dataframe=dataframe, fast_length=indicators_dict["MACD"]["fast_length"], slow_length=indicators_dict["MACD"]["slow_length"],
-            #                       signal_length=indicators_dict["MACD"]["signal_smoothing"], source=indicators_dict["MACD"]["source"])
             dataframe["macd"] = round(macd_df.iloc[:, 0], 2).fillna(0).values
             dataframe["macd_h"] = round(macd_df.iloc[:, 1], 2).fillna(0).values
             dataframe["macd_s"] = round(macd_df.iloc[:, 2], 2).fillna(0).values
@@ -673,10 +686,6 @@ def apply_indicators(dataframe, indicators_dict):
             dataframe["smi_s"] = round(smi_df.iloc[:, 1], 2).fillna(0).values
             dataframe["smi_o"] = round(smi_df.iloc[:, 2], 2).fillna(0).values
 
-        if "MCGD" in indicators_dict:
-            mcgd_df = TA.mcgd(close=dataframe["close"], length=indicators_dict["MCGD"]["length"])
-            dataframe["mcgd"] = round(mcgd_df.iloc[:], 2).fillna(0).values
-
         if "TPR" in indicators_dict:
             dataframe["tpr"] = round((dataframe["high"] + dataframe["low"] + dataframe["close"])/3, 2)
 
@@ -838,9 +847,12 @@ def apply_indicators(dataframe, indicators_dict):
             slope_df = calculate_slope(dataframe=dataframe, source=indicators_dict["SLOPE"]["source"])
             dataframe[f'{indicators_dict["SLOPE"]["source"]}_slope'] = slope_df.iloc[:, 0]
 
-        # if "MCGD" in indicators_dict:
-        #     mcgd_df = mcgd(close=dataframe["close"], length=indicators_dict["MCGD"]["length"])
-        #     dataframe["mcgd"] = round(mcgd_df, 2).fillna(0).values
+        if "MCGD" in indicators_dict:
+            mcgd_df = mcgd(close=dataframe["close"], length=indicators_dict["MCGD"]["length"])
+            dataframe["mcgd"] = round(mcgd_df, 2).fillna(0).values
+
+            # mcgd_df = TA.mcgd(close=dataframe["close"], length=indicators_dict["MCGD"]["length"])
+            # dataframe["mcgd"] = round(mcgd_df.iloc[:], 2).fillna(0).values
 
         if "AO" in indicators_dict:
             ao_df = TA.ao(high=dataframe["high"], low=dataframe["low"])
@@ -848,6 +860,25 @@ def apply_indicators(dataframe, indicators_dict):
             dataframe["ao"] = round(ao_df, 2).fillna(0).values
             dataframe["ao_color"] = np.where(ao_diff > 0, "green", "red")
 
+        if "WMA" in indicators_dict:
+            wma_df = TA.wma(close=dataframe["close"], length=indicators_dict["WMA"]["length"])
+            dataframe["wma"] = round(wma_df, 2).fillna(0).values
+
+        if "WMA9" in indicators_dict:
+            wma_df = TA.wma(close=dataframe["close"], length=indicators_dict["WMA9"]["length"])
+            dataframe["wma9"] = round(wma_df, 2).fillna(0).values
+
+        if "WMA30" in indicators_dict:
+            wma_df = TA.wma(close=dataframe["close"], length=indicators_dict["WMA30"]["length"])
+            dataframe["wma30"] = round(wma_df, 2).fillna(0).values
+
+        if "VOLMA" in indicators_dict:
+            if indicators_dict["VOLMA"]["mamode"] == "ema":
+                volma_df = TA.ema(close=dataframe["volume"], length=indicators_dict["VOLMA"]["length"])
+            else:
+                volma_df = TA.sma(close=dataframe["volume"], length=indicators_dict["VOLMA"]["length"])
+            dataframe["volma"] = round(volma_df, 2).fillna(0).values
+
         return dataframe
 
     except Exception as e:
@@ -1410,7 +1441,6 @@ def calculate_brokerage(buy_price, sell_price, quantity, broker="zerodha", marke
 
             # Calculating Turn Over
             turnover = (buy_price * quantity) + (sell_price * quantity)
-            print(f"Turnover : {turnover}")
 
             # Calculating Brokerage
             if market_type == "options":
@@ -1440,7 +1470,10 @@ def calculate_brokerage(buy_price, sell_price, quantity, broker="zerodha", marke
             total_charges = round((brokerage + sebi_charges + exchange_charges + stt + clearing_charge + stamp_duty + gst), 2)
 
             # Calculating Net PnL
-            net_pl = ((sell_price - buy_price) * quantity) - total_charges
+            if position_type.upper() == "LONG":
+                net_pl = round(((sell_price - buy_price) * quantity) - total_charges, 2)
+            else:
+                net_pl = round(((buy_price - sell_price) * quantity) - total_charges, 2)
 
             return total_charges, net_pl
 
@@ -1483,7 +1516,10 @@ def calculate_brokerage(buy_price, sell_price, quantity, broker="zerodha", marke
             total_charges = round((brokerage + sebi_charges + exchange_charges + stt + clearing_charge + stamp_duty + gst), 2)
 
             # Calculating Net PnL
-            net_pl = ((sell_price - buy_price) * quantity) - total_charges
+            if position_type.upper() == "LONG":
+                net_pl = round(((sell_price - buy_price) * quantity) - total_charges, 2)
+            else:
+                net_pl = round(((buy_price - sell_price) * quantity) - total_charges, 2)
 
             return total_charges, net_pl
 
@@ -1494,7 +1530,10 @@ def calculate_brokerage(buy_price, sell_price, quantity, broker="zerodha", marke
             buy_charges = round(buy_turnover * brokerage/100, 2)
             sell_charges = round(sell_turnover * brokerage/100, 2)
             total_charges = buy_charges + sell_charges
-            net_pl = ((sell_price - buy_price) * quantity) - total_charges
+            if position_type.upper() == "LONG":
+                net_pl = round(((sell_price - buy_price) * quantity) - total_charges, 2)
+            else:
+                net_pl = round(((buy_price - sell_price) * quantity) - total_charges, 2)
             return total_charges, net_pl
 
         elif broker == "schwab":
@@ -1504,7 +1543,10 @@ def calculate_brokerage(buy_price, sell_price, quantity, broker="zerodha", marke
             else:  # equities
                 brokerage_fee = 0.0
             total_charges = brokerage_fee
-            net_pl = round(((sell_price - buy_price) * quantity) - total_charges, 2)
+            if position_type.upper() == "LONG":
+                net_pl = round(((sell_price - buy_price) * quantity) - total_charges, 2)
+            else:
+                net_pl = round(((buy_price - sell_price) * quantity) - total_charges, 2)
             return total_charges, net_pl
 
         elif broker == "binance":
